@@ -52,9 +52,8 @@ static map<int, unsigned char> mupsksec2pri;
 
 bool usingFEC = false;
 
-void dominoex::tx_init(SoundBase *sc)
+void dominoex::tx_init()
 {
-	scard = sc;
 	txstate = TX_STATE_PREAMBLE;
 	txprevtone = 0;
 	Mu_bitstate = 0;
@@ -134,6 +133,16 @@ void dominoex::init()
 //	reset_filters();
 	rx_init();
 
+	if (progdefaults.StartAtSweetSpot)
+		set_freq(progdefaults.PSKsweetspot);
+	else if (progStatus.carrier != 0) {
+		set_freq(progStatus.carrier);
+#if !BENCHMARK_MODE
+		progStatus.carrier = 0;
+#endif
+	} else
+		set_freq(wf->Carrier());
+
 	set_scope_mode(Digiscope::DOMDATA);
 }
 
@@ -697,8 +706,9 @@ void dominoex::flushtx()
 
 int dominoex::tx_process()
 {
-	int i;
+	modem::tx_process();
 
+	int i;
 	switch (txstate) {
 	case TX_STATE_PREAMBLE:
 		if (progdefaults.DOMINOEX_FEC)
