@@ -46,6 +46,7 @@
 #include "status.h"
 #include "icons.h"
 #include "threads.h"
+#include "strutil.h"
 
 #include "fileselect.h"
 
@@ -135,12 +136,6 @@ static string trim(string s)
 	return s;
 }
 
-static string ucasestr(string s)
-{
-	for (size_t n = 0; n < s.length(); n++) s[n] = toupper(s[n]);
-	return s;
-}
-
 //--------------------------------------------------------------------------------
 //          1         2         3         4         5         6         7
 //01234567890123456789012345678901234567890123456789012345678901234567890123456789
