@@ -166,7 +166,7 @@ public:
 	~dominoex ();
 	void	init();
 	void	rx_init();
-	void	tx_init(SoundBase *sc);
+	void	tx_init();
 	void	restart();
 	int		rx_process(const double *buf, int len);
 	int		tx_process();
