@@ -526,7 +526,7 @@ extern void updateOutSerNo();
 extern void connect_to_log_server(void *p = NULL);
 extern void set_server_label(bool);
 extern void activate_menu_item(const char *caption, bool val);
-extern bool xml_check_dup();
+extern int  xml_check_dup();
 extern bool xml_get_record(const char *);
 
 extern const char* zdate(void);
