@@ -23,7 +23,7 @@
 
 #include <string>
 
-bool request_reply(const std::string& node, const std::string& service,
+bool network_query(const std::string& node, const std::string& service,
 		   const std::string& request, std::string& reply, double timeout = 0.0);
 bool fetch_http(const std::string& url, std::string& reply, double timeout = 0.0);
 bool fetch_http_gui(const std::string& url, std::string& reply, double timeout = 0.0);
