@@ -42,7 +42,7 @@ public:
 	void	clear();
 	void	setbw(double w) { bw = w;}
 	void	sigdensity();
-	double	sigpeak(int &f, int f1, int f2, int bw);
+	double	sigpeak(int &f, int f1, int f2);
 	double	peak(int &f, int f1, int f2, double level);
 	double	power(int f1, int f2);
 };
