@@ -242,7 +242,7 @@ public:
 	~rtty();
 	void init();
 	void rx_init();
-	void tx_init(SoundBase *sc);
+	void tx_init();
 	void restart();
 	void reset_filters();
 	int rx_process(const double *buf, int len);
