@@ -105,12 +105,6 @@ public:
 	double Ampspan() {
 		return ampspan;
 	}
-	void Reflevel(double RefLev) {
-		reflevel = (int)RefLev;
-	}
-	double Reflevel() {
-		return reflevel;
-	}
 	void Bandwidth (int bw) {
 		bandwidth = bw;
 		makeMarker();
@@ -148,7 +142,8 @@ public:
 	void makeMarker();
 	void process_analog(wf_fft_type *sig, int len);
 	void processFFT();
-	void sig_data( double *sig, int len, int sr );
+	void sig_data( double *sig, int sr);
+	void handle_sig_data();
 	void rfcarrier(long long f) {
 		rfc = f;
 	}
@@ -195,7 +190,6 @@ private:
 	float	dispdec;
 	int 	ampspan;
     double  peakaudio;
-	int 	reflevel;
 	double	dfreq;
 	bool	centercarrier;
 	bool	cursormoved;
@@ -258,8 +252,7 @@ class waterfall: public Fl_Group {
 	friend void center_cb(Fl_Widget *w, void *v);
 	friend void carrier_cb(Fl_Widget *w, void *v);
 	friend void mode_cb(Fl_Widget *w, void *v);
-   friend void set_wf_mode(void);
-	friend void reflevel_cb(Fl_Widget *w, void *v);
+    friend void set_wf_mode(void);
 	friend void ampspan_cb(Fl_Widget *w, void *v);
 	friend void qsy_cb(Fl_Widget *w, void *v);
 	friend void rate_cb(Fl_Widget *w, void *v);
@@ -269,9 +262,10 @@ public:
 	~waterfall(){};
 	void show_scope(bool on);
 	void opmode();
-	void sig_data(double *sig, int len, int sr){
-		wfdisp->sig_data(sig, len, sr);
+	void sig_data(double *sig, int sr) {
+		wfdisp->sig_data(sig, sr);
 	}
+	void handle_sig_data() {wfdisp->handle_sig_data();}
 	void Overload(bool ovr) {
 		wfdisp->Overload(ovr);
 	}
@@ -358,11 +352,9 @@ public:
 	Digiscope	*wfscope;
 
 private:
-	//bool		ishidden;
 	int			wf_dim;
-	//Fl_Box		*bezel;
 	WFdisp		*wfdisp;
-	Fl_Group	*rs1; //, *rs2, *hidden;
+	Fl_Group	*rs1;
 	Fl_Button	*mode;
 	Fl_Button	*x1;
 	Fl_Button	*left;
