@@ -75,9 +75,8 @@ char mfskmsg[80];
 
 #include "mfsk-pic.cxx"
 
-void  mfsk::tx_init(SoundBase *sc)
+void  mfsk::tx_init()
 {
-	scard = sc;
 	txstate = TX_STATE_PREAMBLE;
 	bitstate = 0;
 
@@ -128,6 +127,17 @@ void mfsk::init()
 	setpicture_link(this);
 	TXspp = txSPP;
 	RXspp = 8;
+
+	if (progdefaults.StartAtSweetSpot)
+		set_freq(progdefaults.PSKsweetspot);
+	else if (progStatus.carrier != 0) {
+		set_freq(progStatus.carrier);
+#if !BENCHMARK_MODE
+		progStatus.carrier = 0;
+#endif
+	} else
+		set_freq(wf->Carrier());
+
 }
 
 void mfsk::shutdown()
@@ -1136,6 +1146,7 @@ int mfsk::tx_process()
 		case TX_STATE_PICTURE:
 			int i = 0;
 			int blocklen = 128;
+			stop_deadman();
 			while (i < xmtbytes) {
 				if (stopflag || abortxmt)
 					break;
@@ -1152,6 +1163,7 @@ int mfsk::tx_process()
 				i += blocklen;
 			}
 			flushtx(preamble);
+			start_deadman();
 
 			REQ_FLUSH(GET_THREAD_ID());
 
