@@ -27,6 +27,8 @@
 
 #include <config.h>
 
+#include <sstream>
+
 #include <FL/Fl.H>
 
 #include "olivia.h"
@@ -59,9 +61,8 @@ double olivia::nco(double freq)
 	return cos(preamblephase);
 }
 
-void olivia::tx_init(SoundBase *sc)
+void olivia::tx_init()
 {
-	scard = sc;
 	phaseacc = 0;
 	prevsymbol = cmplx (1.0, 0.0);
 	preamble = 32;
@@ -160,6 +161,8 @@ int olivia::unescape(int c)
 
 int olivia::tx_process()
 {
+	modem::tx_process();
+
 	int c = 0, len = 0;
 
 	if ((mode == MODE_OLIVIA && 
@@ -379,10 +382,12 @@ void olivia::restart()
 	fragmentsize = 1024;
 	set_bandwidth(Tx->Bandwidth - Tx->Bandwidth / Tx->Tones);
 
+	stringstream info;
 	if (mode == MODE_OLIVIA)
-		put_MODEstatus("%s %" PRIuSZ "/%" PRIuSZ "", get_mode_name(), Tx->Tones, Tx->Bandwidth);
+		info << get_mode_name() << " " << Tx->Tones << "/" << Tx->Bandwidth;
 	else
-		put_MODEstatus("%s", mode_info[mode].sname);//get_mode_name());
+		info << mode_info[mode].sname;
+	put_MODEstatus("%s", info.str().c_str());
 
 	metric = 0;
 
@@ -396,6 +401,17 @@ void olivia::init()
 	restart();
 	modem::init();
 	set_scope_mode(Digiscope::BLANK);
+
+	if (progdefaults.StartAtSweetSpot)
+		set_freq(progdefaults.PSKsweetspot);
+	else if (progStatus.carrier != 0) {
+		set_freq(progStatus.carrier);
+#if !BENCHMARK_MODE
+		progStatus.carrier = 0;
+#endif
+	} else
+		set_freq(wf->Carrier());
+
 }
 
 olivia::olivia(trx_mode omode)
