@@ -5,8 +5,9 @@
 //		Dave Freese, W1HKJ
 // Copyright (C) 2009-2010
 //		John Douyere, VK2ETA
-// Copyright (C) 2014
+// Copyright (C) 2014-2018
 //		John Phelps, KL4YFD
+//      Modified by Joe Counsil, K0OG - Flushlengths on 8PSKxF Modes
 //
 // PSK-FEC and PSK-R modes contributed by VK2ETA
 //
@@ -52,6 +53,7 @@
 #include "macros.h"
 
 #include "confdialog.h"
+#include "test_signal.h"
 
 extern waterfall *wf;
 
@@ -120,9 +122,8 @@ static unsigned char graymapped_8psk_softbits[8][3] =  {
 
 char pskmsg[80];
 
-void psk::tx_init(SoundBase *sc)
+void psk::tx_init()
 {
-	scard = sc;
 	for (int car = 0; car < numcarriers; car++) {
 		phaseacc[car] = 0;
 		prevsymbol[car] = cmplx (1.0, 0.0);
@@ -140,7 +141,6 @@ void psk::tx_init(SoundBase *sc)
 
 	// interleaver
 	bitshreg = 0;
-	startpreamble = true;
 
 	symbols = 0;
 	acc_symbols = 0;
@@ -155,12 +155,6 @@ void psk::tx_init(SoundBase *sc)
 	vphase = 0;
 	maxamp = 0;
 
-	double bw2 = 6.0 * bandwidth;
-	double flo = (get_txfreq_woffset() - bw2);
-	if (flo <= 0) flo = 0;
-	double fhi = (get_txfreq_woffset() + bw2);
-	if (fhi >= 0.48*samplerate) fhi = 0.48*samplerate;
-	xmtfilt->init_bandpass (127, 1, flo/samplerate, fhi/samplerate);
 }
 
 void psk::rx_init()
@@ -194,13 +188,26 @@ void psk::rx_init()
 	if (Rxinlv) Rxinlv->flush();
 	if (Rxinlv2) Rxinlv2->flush();
 
+	for (int i = 0; i < NUM_FILTERS; i++) {
+		re_Gbin[i]->reset();
+		im_Gbin[i]->reset();
+	}
 }
 
+bool psk::viewer_mode()
+{
+	if (mode == MODE_PSK31 || mode == MODE_PSK63 || mode == MODE_PSK63F ||
+		mode == MODE_PSK125 || mode == MODE_PSK250 || mode == MODE_PSK500 ||
+		mode == MODE_PSK125R || mode == MODE_PSK250R || mode == MODE_PSK500R ||
+		mode == MODE_QPSK31 || mode == MODE_QPSK63 || mode == MODE_QPSK125 ||
+		mode == MODE_QPSK250 || mode == MODE_QPSK500 )
+		return true;
+	return false;
+}
 
 void psk::restart()
 {
-	if ((mode >= MODE_PSK31 && mode <= MODE_PSK125) ||
-		(mode >= MODE_QPSK31 && mode <= MODE_QPSK125))
+	if (viewer_mode())
 		pskviewer->restart(mode);
 	evalpsk->setbw(sc_bw);
 }
@@ -214,11 +221,22 @@ void psk::init()
 	snratio = 1.0;
 	imdratio = 0.001;
 	rx_init();
+
+	if (progdefaults.StartAtSweetSpot)
+		set_freq(progdefaults.PSKsweetspot);
+	else if (progStatus.carrier != 0) {
+		set_freq(progStatus.carrier);
+#if !BENCHMARK_MODE
+		progStatus.carrier = 0;
+#endif
+	} else
+		set_freq(wf->Carrier());
 }
 
 psk::~psk()
 {
 	if (tx_shape) delete [] tx_shape;
+	if (imd_shape) delete [] imd_shape;
 	if (enc) delete enc;
 	if (dec) delete dec;
 	// FEC 2nd Viterbi decoder
@@ -228,11 +246,16 @@ psk::~psk()
 		if (fir1[i]) delete fir1[i];
 		if (fir2[i]) delete fir2[i];
 	}
-	if (snfilt) delete snfilt;
-	if (imdfilt) delete imdfilt;
+
 	if (e0_filt) delete e0_filt;
 	if (e1_filt) delete e1_filt;
 	if (e2_filt) delete e2_filt;
+	if (e3_filt) delete e3_filt;
+
+	for (int i = 0; i < NUM_FILTERS; i++) {
+		delete re_Gbin[i];
+		delete im_Gbin[i];
+	}
 
 	if (pskviewer) delete pskviewer;
 	if (evalpsk) delete evalpsk;
@@ -244,11 +267,13 @@ psk::~psk()
 
 	if (vestigial_sfft) delete vestigial_sfft;
 
-	if (xmtfilt) delete xmtfilt;
 }
 
 psk::psk(trx_mode pskmode) : modem()
 {
+enum FIR_TYPE {PSK_CORE, GMFSK, SINC};
+FIR_TYPE fir_type = PSK_CORE;
+
 	cap |= CAP_AFC | CAP_AFC_SR;
 
 	mode = pskmode;
@@ -269,26 +294,32 @@ psk::psk(trx_mode pskmode) : modem()
 		case MODE_PSK31:
 			symbollen = 256;
 			dcdbits = 32;
+			fir_type = PSK_CORE;
 			break;
 		case MODE_PSK63:
 			symbollen = 128;
 			dcdbits = 64;
+			fir_type = PSK_CORE;
 			break;
 		case MODE_PSK125:
 			symbollen = 64;
 			dcdbits = 128;
+			fir_type = SINC;
 			break;
 		case MODE_PSK250:
 			symbollen = 32;
 			dcdbits = 256;
+			fir_type = SINC;
 			break;
 		case MODE_PSK500:
 			symbollen = 16;
 			dcdbits = 512;
+			fir_type = SINC;
 			break;
 		case MODE_PSK1000:
 			symbollen = 8;
 			dcdbits = 128;
+			fir_type = SINC;
 			break;
 
 		case MODE_QPSK31:
@@ -296,39 +327,45 @@ psk::psk(trx_mode pskmode) : modem()
 			_qpsk = true;
 			dcdbits = 32;
 			cap |= CAP_REV;
+			fir_type = PSK_CORE;
 			break;
 		case MODE_QPSK63:
 			symbollen = 128;
 			_qpsk = true;
 			dcdbits = 64;
 			cap |= CAP_REV;
+			fir_type = PSK_CORE;
 			break;
 		case MODE_QPSK125:
 			symbollen = 64;
 			_qpsk = true;
 			dcdbits = 128;
 			cap |= CAP_REV;
+			fir_type = SINC;
 			break;
 		case MODE_QPSK250:
 			symbollen = 32;
 			_qpsk = true;
 			dcdbits = 256;
 			cap |= CAP_REV;
+			fir_type = SINC;
 			break;
 		case MODE_QPSK500:
 			symbollen = 16;
 			_qpsk = true;
 			dcdbits = 512;
 			cap |= CAP_REV;
+			fir_type = SINC;
 			break;
 		case MODE_PSK63F:  // As per Multipsk (BPSK63 + FEC + MFSK Varicode)
 			symbollen = 128;
 			_pskr = true;
 			dcdbits = 64;
+			fir_type = PSK_CORE;
 			break;
 
 	// 8psk modes without FEC
-		case MODE_8PSK125:
+		case MODE_8PSK125: // 125 baud | 375 bits/sec No FEC
 			symbollen = 128;
 			samplerate = 16000;
 			_8psk = true;
@@ -336,8 +373,9 @@ psk::psk(trx_mode pskmode) : modem()
 			dcdbits = 128;
 			vestigial = true;
 			cap |= CAP_REV;
+			fir_type = SINC;
 			break;
-		case MODE_8PSK250: // 250 baud | 375 bits/sec @ 1/2 Rate FEC
+		case MODE_8PSK250: // 250 baud | 750 bits/sec No FEC
 			symbollen = 64;
 			samplerate = 16000;
 			_8psk = true;
@@ -345,8 +383,9 @@ psk::psk(trx_mode pskmode) : modem()
 			dcdbits = 256;
 			vestigial = true;
 			cap |= CAP_REV;
+			fir_type = SINC;
 			break;
-		case MODE_8PSK500: // 500 baud | 1000 bits/sec @ 2/3 rate FEC
+		case MODE_8PSK500: // 500 baud | 1500 bits/sec No FEC
 			symbollen = 32;
 			samplerate = 16000;
 			_8psk = true;
@@ -354,6 +393,7 @@ psk::psk(trx_mode pskmode) : modem()
 			dcdbits = 512;
 			vestigial = true;
 			cap |= CAP_REV;
+			fir_type = SINC;
 			break;
 		case MODE_8PSK1000: // 1000 baud | 3000 bits/sec  No FEC
 			symbollen = 16;
@@ -363,64 +403,70 @@ psk::psk(trx_mode pskmode) : modem()
 			dcdbits = 1024;
 			vestigial = true;
 			cap |= CAP_REV;
+			fir_type = SINC;
 			break;
 
 	// 8psk modes with FEC
-		case MODE_8PSK125FL:
+		case MODE_8PSK125FL: // 125 baud | 187 bits/sec @ 1/2 Rate K=13 FEC
 			symbollen = 128;
-			idepth = 384; // 1024 milliseconds
-			flushlength = 38;
+			idepth = 384; // 2048 milliseconds
+			flushlength = 55;
 			samplerate = 16000;
 			_8psk = true;
 			dcdbits = 128;
 			vestigial = true;
 			cap |= CAP_REV;
+			fir_type = SINC;
 			break;
-		case MODE_8PSK250FL: // 250 baud | 375 bits/sec @ 1/2 Rate FEC
+		case MODE_8PSK250FL: // 250 baud | 375 bits/sec @ 1/2 Rate K=13 FEC
 			symbollen = 64;
-			idepth = 512; // 682 milliseconds
-			flushlength = 47;
+			idepth = 512; // 1365 milliseconds
+			flushlength = 65;
 			samplerate = 16000;
 			_8psk = true;
 			dcdbits = 256;
 			vestigial = true;
 			cap |= CAP_REV;
+			fir_type = SINC;
 			break;
-		case MODE_8PSK125F:
+		case MODE_8PSK125F: // 125 baud | 187 bits/sec @ 1/2 Rate K=16 FEC
 			symbollen = 128;
-			idepth = 384; // 1024 milliseconds
-			flushlength = 38;
+			idepth = 384; // 2048 milliseconds
+			flushlength = 55;
 			samplerate = 16000;
 			_8psk = true;
 			dcdbits = 128;
 			vestigial = true;
 			cap |= CAP_REV;
+			fir_type = SINC;
 			break;
-		case MODE_8PSK250F: // 250 baud | 375 bits/sec @ 1/2 Rate FEC
+		case MODE_8PSK250F: // 250 baud | 375 bits/sec @ 1/2 Rate K=16 FEC
 			symbollen = 64;
-			idepth = 512; // 682 milliseconds
-			flushlength = 47;
+			idepth = 512; // 1365 milliseconds
+			flushlength = 65;
 			samplerate = 16000;
 			_8psk = true;
 			dcdbits = 256;
 			vestigial = true;
 			cap |= CAP_REV;
+			fir_type = SINC;
 			break;
-		case MODE_8PSK500F: // 500 baud | 1000 bits/sec @ 2/3 rate FEC
+		case MODE_8PSK500F: // 500 baud | 1000 bits/sec @ 2/3 rate K=13 FEC
 			symbollen = 32;
 			idepth = 640; // 426 milliseconds
-			flushlength = 62;
+			flushlength = 80;
 			samplerate = 16000;
 			_8psk = true;
 			_puncturing = true;
 			dcdbits = 512;
 			vestigial = true;
 			cap |= CAP_REV;
+			fir_type = SINC;
 			break;
-		case MODE_8PSK1000F: // 1000 baud | 3000 bits/sec
+		case MODE_8PSK1000F: // 1000 baud | 2000 bits/sec @ 2/3 rate K=7 FEC
 			symbollen = 16;
-			idepth = 512;
-			flushlength = 56;
+			idepth = 512; // 170 milliseconds
+			flushlength = 120;
 			samplerate = 16000;
 			_8psk = true;
 			dcdbits = 1024;
@@ -428,11 +474,12 @@ psk::psk(trx_mode pskmode) : modem()
 			_puncturing = true;
 			vestigial = true;
 			PSKviterbi = true;
+			fir_type = SINC;
 			break;
-		case MODE_8PSK1200F: // 1200 baud | 1800 bits/sec
+		case MODE_8PSK1200F: // 1200 baud | 2400 bits/sec @ 2/3 rate K=7 FEC
 			symbollen = 13;
-			idepth = 512;
-			flushlength = 56;
+			idepth = 512; // 142 milliseconds
+			flushlength = 175;
 			samplerate = 16000;
 			_8psk = true;
 			_puncturing = true;
@@ -440,6 +487,7 @@ psk::psk(trx_mode pskmode) : modem()
 			cap |= CAP_REV;
 			vestigial = true;
 			PSKviterbi = true;
+			fir_type = SINC;
 			break;
 			// end 8psk modes
 
@@ -448,24 +496,28 @@ psk::psk(trx_mode pskmode) : modem()
 			_pskr = true;
 			dcdbits = 128;
 			idepth = 40;  // 2x2x40 interleaver
+			fir_type = SINC;
 			break;
 		case MODE_PSK250R:
 			symbollen = 32;
 			_pskr = true;
 			dcdbits = 256;
 			idepth = 80;  // 2x2x80 interleaver
+			fir_type = SINC;
 			break;
 		case MODE_PSK500R:
 			symbollen = 16;
 			_pskr = true;
 			dcdbits = 512;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 		case MODE_PSK1000R:
 			symbollen = 8;
 			_pskr = true;
 			dcdbits = 512;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 
 			// multi-carrier modems
@@ -475,6 +527,7 @@ psk::psk(trx_mode pskmode) : modem()
 			_pskr = true;//PSKR
 			numcarriers = 4;
 			idepth = 80; // 2x2x80 interleaver
+			fir_type = SINC;
 			break;
 		case MODE_5X_PSK63R:
 			symbollen = 128; //PSK63
@@ -482,6 +535,7 @@ psk::psk(trx_mode pskmode) : modem()
 			_pskr = true; //PSKR
 			numcarriers = 5;
 			idepth = 260; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 		case MODE_10X_PSK63R:
 			symbollen = 128; //PSK63
@@ -489,6 +543,7 @@ psk::psk(trx_mode pskmode) : modem()
 			_pskr = true; //PSKR
 			numcarriers = 10;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 		case MODE_20X_PSK63R:
 			symbollen = 128; //PSK63
@@ -496,6 +551,7 @@ psk::psk(trx_mode pskmode) : modem()
 			_pskr = true; //PSKR
 			numcarriers = 20;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 		case MODE_32X_PSK63R:
 			symbollen = 128; //PSK63
@@ -503,6 +559,7 @@ psk::psk(trx_mode pskmode) : modem()
 			_pskr = true; //PSKR
 			numcarriers = 32;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 
 		case MODE_4X_PSK125R:
@@ -511,6 +568,7 @@ psk::psk(trx_mode pskmode) : modem()
 			_pskr = true;//PSKR
 			numcarriers = 4;
 			idepth = 80; // 2x2x80 interleaver
+			fir_type = SINC;
 			break;
 		case MODE_5X_PSK125R:
 			symbollen = 64;//PSK125
@@ -518,6 +576,7 @@ psk::psk(trx_mode pskmode) : modem()
 			_pskr = true;//PSKR
 			numcarriers = 5;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 		case MODE_10X_PSK125R:
 			symbollen = 64;//PSK125
@@ -525,12 +584,14 @@ psk::psk(trx_mode pskmode) : modem()
 			_pskr = true;//PSKR
 			numcarriers = 10;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 
 		case MODE_12X_PSK125:
 			symbollen = 64;//PSK125
 			dcdbits = 128;//512;
 			numcarriers = 12;
+			fir_type = SINC;
 			break;
 		case MODE_12X_PSK125R:
 			symbollen = 64;//PSK125
@@ -538,6 +599,7 @@ psk::psk(trx_mode pskmode) : modem()
 			_pskr = true;//PSKR
 			numcarriers = 12;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 
 		case MODE_16X_PSK125R:
@@ -546,6 +608,7 @@ psk::psk(trx_mode pskmode) : modem()
 			_pskr = true;//PSKR
 			numcarriers = 16;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 
 		case MODE_2X_PSK250R:
@@ -554,6 +617,7 @@ psk::psk(trx_mode pskmode) : modem()
 			_pskr = true;//PSKR
 			numcarriers = 2;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 		case MODE_3X_PSK250R:
 			symbollen = 32;//PSK250
@@ -561,6 +625,7 @@ psk::psk(trx_mode pskmode) : modem()
 			_pskr = true;//PSKR
 			numcarriers = 3;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 		case MODE_5X_PSK250R:
 			symbollen = 32;//PSK250
@@ -568,11 +633,13 @@ psk::psk(trx_mode pskmode) : modem()
 			dcdbits = 1024;
 			numcarriers = 5;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 		case MODE_6X_PSK250:
 			symbollen = 32;//PSK250
 			dcdbits = 512;
 			numcarriers = 6;
+			fir_type = SINC;
 			break;
 		case MODE_6X_PSK250R:
 			symbollen = 32;//PSK250
@@ -580,6 +647,7 @@ psk::psk(trx_mode pskmode) : modem()
 			dcdbits = 1024;
 			numcarriers = 6;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 		case MODE_7X_PSK250R:
 			symbollen = 32;//PSK250
@@ -587,17 +655,20 @@ psk::psk(trx_mode pskmode) : modem()
 			dcdbits = 1024;
 			numcarriers = 7;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 
 		case MODE_2X_PSK500:
 			symbollen = 16;
 			dcdbits = 512;
 			numcarriers = 2;
+			fir_type = SINC;
 			break;
 		case MODE_4X_PSK500:
 			symbollen = 16;
 			dcdbits = 512;
 			numcarriers = 4;
+			fir_type = SINC;
 			break;
 
 		case MODE_2X_PSK500R:
@@ -606,6 +677,7 @@ psk::psk(trx_mode pskmode) : modem()
 			dcdbits = 1024;
 			idepth = 160; // 2x2x160 interleaver
 			numcarriers = 2;
+			fir_type = SINC;
 			break;
 		case MODE_3X_PSK500R:
 			symbollen = 16;
@@ -613,6 +685,7 @@ psk::psk(trx_mode pskmode) : modem()
 			dcdbits = 1024;
 			idepth = 160; // 2x2x160 interleaver
 			numcarriers = 3;
+			fir_type = SINC;
 			break;
 		case MODE_4X_PSK500R:
 			symbollen = 16;
@@ -620,6 +693,7 @@ psk::psk(trx_mode pskmode) : modem()
 			dcdbits = 1024;
 			idepth = 160; // 2x2x160 interleaver
 			numcarriers = 4;
+			fir_type = SINC;
 			break;
 
 		case MODE_2X_PSK800:
@@ -627,6 +701,7 @@ psk::psk(trx_mode pskmode) : modem()
 			_pskr = false;
 			dcdbits = 512;
 			numcarriers = 2;
+			fir_type = SINC;
 			break;
 		case MODE_2X_PSK800R:
 			symbollen = 10;
@@ -634,6 +709,7 @@ psk::psk(trx_mode pskmode) : modem()
 			dcdbits = 1024;
 			idepth = 160; // 2x2x160 interleaver
 			numcarriers = 2;
+			fir_type = SINC;
 			break;
 
 		case MODE_2X_PSK1000:
@@ -641,6 +717,7 @@ psk::psk(trx_mode pskmode) : modem()
 			dcdbits = 1024;
 			numcarriers = 2;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 		case MODE_2X_PSK1000R:
 			symbollen = 8;//PSK1000
@@ -648,6 +725,7 @@ psk::psk(trx_mode pskmode) : modem()
 			dcdbits = 1024;
 			numcarriers = 2;
 			idepth = 160; // 2x2x160 interleaver
+			fir_type = SINC;
 			break;
 
 		default:
@@ -655,6 +733,7 @@ psk::psk(trx_mode pskmode) : modem()
 			symbollen = 256;
 			dcdbits = 32;
 			numcarriers = 1;
+			fir_type = PSK_CORE;
 	}
 
 	// Set the number of bits-per-symbol based on the chosen constellation
@@ -679,8 +758,8 @@ psk::psk(trx_mode pskmode) : modem()
 	vestigial_sfft = (sfft *)0;
 
 	// create impulse response for experimental FIR filters
-	double fir1c[64];
-	double fir2c[64];
+	double fir1c[FIRLEN+1];
+	double fir2c[FIRLEN+1];
 
 	for (int i = 0; i < MAX_CARRIERS; i++) {
 		if (i < numcarriers) {
@@ -692,61 +771,41 @@ psk::psk(trx_mode pskmode) : modem()
 		}
 	}
 
-	switch (progdefaults.PSK_filter) {
-		case 1:
-			// use the original gmfsk matched filters
-			for (int i = 0; i < 64; i++) {
-				fir1c[i] = gmfir1c[i];
-				fir2c[i] = gmfir2c[i];
+	switch (fir_type) {
+		case PSK_CORE: // PSKcore filter
+			raisedcosfilt(fir1c, FIRLEN);
+			for (int i = 0; i <= FIRLEN; i++)
+				fir2c[i] = pskcore_filter[i];
+			for (int i = 0; i < numcarriers; i++) {
+				fir1[i]->init(FIRLEN+1, symbollen > 15 ? symbollen / 16 : 1, fir1c, fir1c);
+				fir2[i]->init(FIRLEN+1, 1, fir2c, fir2c);
 			}
+			break;
+		default:
+		case SINC: // fir1c & fir2c matched sin(x)/x filter w blackman
+			wsincfilt(fir1c, 1.0 / symbollen, FIRLEN);
+			wsincfilt(fir2c, 1.0 / 16.0, FIRLEN);
 			for (int i = 0; i < numcarriers; i++) {
 				fir1[i]->init(FIRLEN, symbollen > 15 ? symbollen / 16 : 1, fir1c, fir1c);
 				fir2[i]->init(FIRLEN, 1, fir2c, fir2c);
 			}
 			break;
-		case 2:
-			// creates fir1c matched sin(x)/x filter w hamming
-			wsincfilt(fir1c, 1.0 / symbollen, false);
-			// creates fir2c matched sin(x)/x filter w hamming
-			wsincfilt(fir2c, 1.0 / 16.0, false);
-			for (int i = 0; i < numcarriers; i++) {
-				fir1[i]->init(FIRLEN, symbollen > 15 ? symbollen / 16 : 1, fir1c, fir1c);
-				fir2[i]->init(FIRLEN, 1, fir2c, fir2c);
-			}
-			break;
-		case 3:
-			// creates fir1c matched sin(x)/x filter w hamming
-			wsincfilt(fir1c, 1.0 / symbollen, false);
-			// 1/22 with Hamming window nearly identical to gmfir2c
-			wsincfilt(fir2c, 1.0 / 22.0, false);
-			for (int i = 0; i < numcarriers; i++) {
-				fir1[i]->init(FIRLEN, symbollen > 15 ? symbollen / 16 : 1, fir1c, fir1c);
-				fir2[i]->init(FIRLEN, 1, fir2c, fir2c);
-			}
-			break;
-		case 4:
-			wsincfilt(fir2c, 1.5 / 16.0, true);
-			for (int i = 0; i < numcarriers; i++) {
-				fir1[i]->init_lowpass (FIRLEN, 16, 1.5 / symbollen);
-				fir2[i]->init(FIRLEN, 1, fir2c, fir2c);
-			}
-		case 0:
-		default :
-			// creates fir1c matched sin(x)/x filter w blackman
-			wsincfilt(fir1c, 1.0 / symbollen, true);
-			// creates fir2c matched sin(x)/x filter w blackman
-			wsincfilt(fir2c, 1.0 / 16.0, true);
-			for (int i = 0; i < numcarriers; i++) {
-				fir1[i]->init(FIRLEN, symbollen > 15 ? symbollen / 16 : 1, fir1c, fir1c);
-				fir2[i]->init(FIRLEN, 1, fir2c, fir2c);
-			}
 	}
 
-	snfilt = new Cmovavg(16);
-	imdfilt = new Cmovavg(16);
 	e0_filt = new Cmovavg(dcdbits / 2);
 	e1_filt = new Cmovavg(dcdbits / 2);
 	e2_filt = new Cmovavg(dcdbits / 2);
+	e3_filt = new Cmovavg(dcdbits / 2);
+
+	re_Gbin[0] = new goertzel(160, 0,      500.0);	// base
+	re_Gbin[1] = new goertzel(160, 15.625, 500.0);	// fundamental
+	re_Gbin[2] = new goertzel(160, 62.5,   500.0);	// 4th harmonic (noise)
+	re_Gbin[3] = new goertzel(160, 46.875, 500.0);	// 3rd harmonic (imd)
+
+	im_Gbin[0] = new goertzel(160, 0,      500.0);	// base
+	im_Gbin[1] = new goertzel(160, 15.625, 500.0);	// fundamental
+	im_Gbin[2] = new goertzel(160, 62.5,   500.0);	// 4th harmonic (noise)
+	im_Gbin[3] = new goertzel(160, 46.875, 500.0);	// 3rd harmonic (imd)
 
 	if (_disablefec) {
 		enc = NULL;
@@ -771,7 +830,7 @@ psk::psk(trx_mode pskmode) : modem()
 		enc = new encoder(K16, K16_POLY1, K16_POLY2);
 		dec = new viterbi(K16, K16_POLY1, K16_POLY2);
 		dec->setchunksize(4);
-		dec2 = new viterbi(K13, K16_POLY1, K16_POLY2);
+		dec2 = new viterbi(K16, K16_POLY1, K16_POLY2);
 		dec2->setchunksize(4);
 
 	} else if (_xpsk || _8psk || _16psk) {
@@ -800,13 +859,20 @@ psk::psk(trx_mode pskmode) : modem()
 
 	bitshreg = 0;
 	rxbitstate = 0;
-	startpreamble = true;
 
 	tx_shape = new double[symbollen];
+	imd_shape = new double[symbollen];
 
 	// raised cosine shape for the transmitter
-	for ( int i = 0; i < symbollen; i++)
-		tx_shape[i] = 0.5 * cos(i * M_PI / symbollen) + 0.5;
+	double sym_ph = 0;
+	for ( int i = 0; i < symbollen; i++) {
+		sym_ph = i * M_PI / symbollen;
+		tx_shape[i] = 0.5 * cos(sym_ph) + 0.5;
+		imd_shape[i] = 0.5 * ( cos(3.0 * sym_ph) +
+							   (3.0/5.0) * cos(5.0 * sym_ph) +
+							   (3.0/7.0) * cos(7.0 * sym_ph) +
+							   (3.0/9.0) * cos(9.0 * sym_ph) );
+	}
 
 	fragmentsize = symbollen;
 	sc_bw = samplerate / symbollen;
@@ -822,12 +888,11 @@ psk::psk(trx_mode pskmode) : modem()
 		sigsearch = 0;
 	for (int i = 0; i < 16; i++)
 		syncbuf[i] = 0.0;
-	E1 = E2 = E3 = 0.0;
+//	E1 = E2 = E3 = 0.0;
 	acquire = 0;
 
 	evalpsk = new pskeval;
-	if ((mode >= MODE_PSK31 && mode <= MODE_PSK125) ||
-		(mode >= MODE_QPSK31 && mode <= MODE_QPSK125))
+	if (viewer_mode())
 		pskviewer = new viewpsk(evalpsk, mode);
 	else
 		pskviewer = 0;
@@ -843,8 +908,6 @@ psk::psk(trx_mode pskmode) : modem()
 		for (int i = 0; i < 11; i++) sfft_bins[i] = cmplx(0,0);
 	}
 
-	xmtfilt = new C_FIR_filter();
-
 }
 
 //=============================================================================
@@ -867,7 +930,7 @@ void psk::rx_bit(int bit)
 		// MFSK varicode instead of PSK Varicode
 		if ((shreg & 7) == 1) {
 			c = varidec(shreg >> 1);
-			// Voting at the character level for only PSKR modes
+			// Voting at the character level
 			if (fecmet >= fecmet2 || _disablefec) {
 				if ((c != -1) && (c != 0) && (dcd == true)) {
 					put_rx_char(c);
@@ -1119,6 +1182,7 @@ void psk::findsignal()
 		} else { // normal signal search algorithm
 			f1 = (int)(frequency - progdefaults.SearchRange/2);
 			f2 = (int)(frequency + progdefaults.SearchRange/2);
+			resetSN_IMD();
 			if (evalpsk->sigpeak(ftest, f1, f2) > pow(10, progdefaults.ACQsn / 10.0) ) {
 				frequency = ftest;
 				set_freq(frequency);
@@ -1311,36 +1375,17 @@ void psk::rx_symbol(cmplx symbol, int car)
 
 			// pskr DCD on
 		case 0x0A0A0A0A:
-			if ( _xpsk || _8psk || _16psk) break;
-			if (_qpsk) break;
 			if (!_pskr) break;
 			set_dcdON = 1;
 			break;
 
-			// 8psk DCD on (FEC enabled, with Gray-mapped constellation)
-		case 0x25252525: // UN-punctured
-			if (_pskr || _xpsk || _16psk) break;
-			if (!_8psk) break;
-			if (_disablefec) break;
-			set_dcdON = 1;
-			break;
-		case 0x22222222: // punctured @ 2/3 rate
-			if (_pskr || _xpsk || _16psk) break;
-			if (!_8psk) break;
-			if (_disablefec) break;
-			set_dcdON = 1;
-			break;
-
 		case 0x92492492:	// xpsk DCD off (with FEC disabled)
-			if (_pskr) break;
-			if (_qpsk) break;
 			if (!_xpsk) break;
 			if (!_disablefec) break;
 			set_dcdON = 0;
 			break;
 
 		case 0x10842108:	// 16psk DCD off (with FEC disabled)
-			if (_pskr) break;
 			if (!_16psk) break;
 			if (!_disablefec) break;
 			set_dcdON = 0;
@@ -1352,20 +1397,10 @@ void psk::rx_symbol(cmplx symbol, int car)
 			set_dcdON = 0;
 			break;
 
-		case 0x10410410:	// xpsk DCD on (with FEC enabled)
+		case 0x00000000:	// bpsk DCD off.  8psk DCD on.
 			if (_pskr) break;
-			if (_qpsk) break;
-			if (_8psk) break;
-			if (_16psk) break;
-			if (!_xpsk) break;
-			if (_disablefec) break;
-			set_dcdON = 1;
-			break;
-
-		case 0x00000000:	// bpsk DCD off.  x,8,16psk DCD on (with FEC disabled).
-			if (_pskr) break;
-			if (_xpsk || _8psk || _16psk) {
-				if (!_disablefec) break;
+			if (_xpsk || _16psk) break;
+			if (_8psk) {
 				set_dcdON = 1;
 				break;
 			}
@@ -1380,9 +1415,8 @@ void psk::rx_symbol(cmplx symbol, int car)
 			}
 	}
 
-	displaysn = false;
+	//printf("\n%08x", dcdshreg);
 	if ( 1 == set_dcdON ) {
-		displaysn = true;
 		dcd = true;
 		acquire = 0;
 		quality = cmplx (1.0, 0.0);
@@ -1408,7 +1442,6 @@ void psk::rx_symbol(cmplx symbol, int car)
 			int bitmask = 1;
 			unsigned char xsoftsymbols[symbits];
 
-			//printf("\n");
 			if ( (_puncturing && _16psk) ) rx_pskr(128); // 16psk: recover punctured low bit
 
 			// Soft-decode of Gray-mapped 8psk
@@ -1446,7 +1479,6 @@ void psk::rx_symbol(cmplx symbol, int car)
 				//Hard Decode Section
 				for(int i=0; i<symbits; i++) { // Hard decode symbits into soft-symbols
 					xsoftsymbols[i] = (bits & bitmask) ? 255 : 0 ;
-					//printf(" %.3u ", xsoftsymbols[i]);
 					rx_pskr(xsoftsymbols[i]); // Feed to the PSKR FEC decoder, one bit at a time.
 					bitmask = bitmask << 1;
 				}
@@ -1469,24 +1501,68 @@ void psk::rx_symbol(cmplx symbol, int car)
 	}
 }
 
-static double e0, e1, e2;
+static double e0, e1, e2, e3;
 
 void psk::signalquality()
 {
-//	double e0, e1, e2;
-	e0 = e0_filt->run(m_Energy[0]);
-	e1 = e1_filt->run(m_Energy[1]);
-	e2 = e2_filt->run(m_Energy[2]);
+	double r0 = re_Gbin[0]->mag();
+	double r1 = re_Gbin[1]->mag();
+	double r2 = re_Gbin[2]->mag();
+	double r3 = re_Gbin[3]->mag();
 
-	if (((e0 - e1) > 0) && (e1 > 0))
-		snratio = (e0 - e1) / e1;
-	else
-		snratio = 1000.0;
+	double i0 = im_Gbin[0]->mag();
+	double i1 = im_Gbin[1]->mag();
+	double i2 = im_Gbin[2]->mag();
+	double i3 = im_Gbin[3]->mag();
 
-	if (((e0 - e1) > 0) && ((e2 - e1) > 0) )
-		imdratio = (e2 - e1) / (e0 - e1);
-	else
-		imdratio = 0.001;
+	r0 = sqrtf(r0*r0 + i0*i0);
+	r1 = sqrtf(r1*r1 + i1*i1);
+	r2 = sqrtf(r2*r2 + i2*i2);
+	r3 = sqrtf(r3*r3 + i3*i3);
+//std::cout << r0 << ", " << r1 << ", " << r2 << ", " << r3 << std::endl;
+//	if (r0 > r1) r1 = r0;
+
+	e0 = e0_filt->run(r0);
+	e1 = e1_filt->run(r1);
+	e2 = e2_filt->run(r2);
+	e3 = e3_filt->run(r3);
+
+	if (e1 > e0) {
+		if ((e1 > 2 * e2) && (e2 > 0)) {
+			snratio = e1 / e2;
+			if (snratio < 1.0)
+				snratio = 1.0;
+		} else
+			snratio = 1.0;
+	} else {
+		if ((e0 > 2 * e2) && (e2 > 0)) {
+			snratio = e0 / e2;
+			if (snratio < 1.0) snratio = 1.0;
+		} else
+			snratio = 1.0;
+	}
+
+	if ( (e1 > 2 * e3) && (e3 > 2 * e2) ) {
+		imdratio = e3 / e1;
+		if (imdratio < (1.0 /snratio)) imdratio = 1.0 / snratio;
+	} else
+		imdratio = 1.0 / snratio ;
+
+	displaysn = false;
+	if (snratio > 4)
+		displaysn = true;
+
+	if (r0 > r1) {
+		if ((r0 / r2 < 0.1 * snratio ) || (r0 / r2 < 2.0)) {
+//std::cout << "r0 / r2 " << r0/r2 << ", snratio " << snratio << std::endl;
+			displaysn = false;
+		}
+	} else {
+		if ((r1 / r2 < 0.1 * snratio ) || (r1 / r2 < 2.0)) {
+//std::cout << "r1 / r2 " << r1/r2 << ", snratio " << snratio << std::endl;
+			displaysn = false;
+		}
+	}
 
 }
 
@@ -1497,30 +1573,34 @@ void psk::update_syncscope()
 
 	display_metric(metric);
 
-	if (displaysn) {
+	if (displaysn && mode == MODE_PSK31) {
 		memset(msg1, 0, sizeof(msg1));
 		memset(msg2, 0, sizeof(msg2));
 
 		s2n = 10.0*log10( snratio );
-		snprintf(msg1, sizeof(msg1), "s/n %2.0f dB", s2n);
+		if (s2n < 6)
+			strcpy(msg1, "S/N ---");
+		else
+			snprintf(msg1, sizeof(msg1), "S/N %2.0f dB", s2n);
+
+		put_Status1(
+			msg1,
+			progdefaults.StatusTimeout,
+			progdefaults.StatusDim ? STATUS_DIM : STATUS_CLEAR);
 
 		imd = 10.0*log10( imdratio );
-		snprintf(msg2, sizeof(msg2), "imd %2.0f dB", imd);
 
-		put_Status1(	msg1,
-						progdefaults.StatusTimeout,
-						progdefaults.StatusDim ? STATUS_DIM : STATUS_CLEAR);
-		put_Status2(	msg2,
-						progdefaults.StatusTimeout,
-						progdefaults.StatusDim ? STATUS_DIM : STATUS_CLEAR);
+		if (imd > -10)
+			strcpy(msg2, "IMD ---");
+		else
+			snprintf(msg2, sizeof(msg2), "IMD %2.0f dB", imd);
+
+		put_Status2(
+			msg2,
+			progdefaults.StatusTimeout,
+			progdefaults.StatusDim ? STATUS_DIM : STATUS_CLEAR);
 	}
 
-//static char msg3[50];
-//memset(msg3, 0, sizeof(msg3));
-//snprintf(msg3, sizeof(msg3), "%10.3f, %10.3f, %10.3f",
-//e0, e1, e2);
-//put_status(msg3);
-
 }
 
 char bitstatus[100];
@@ -1531,10 +1611,11 @@ int psk::rx_process(const double *buf, int len)
 	cmplx z, z2[MAX_CARRIERS];
 	bool can_rx_symbol = false;
 
-	if (mode >= MODE_PSK31 && mode <= MODE_PSK125) {
+	if (viewer_mode()) {
 		if (!progdefaults.report_when_visible ||
 			dlgViewer->visible() || progStatus.show_channels )
-			if (pskviewer && !bHistory) pskviewer->rx_process(buf, len);
+			if (pskviewer && !bHistory)
+				pskviewer->rx_process(buf, len);
 		if (evalpsk)
 			evalpsk->sigdensity();
 	}
@@ -1701,7 +1782,7 @@ int psk::rx_process(const double *buf, int len)
 				sigsearch = SIGSEARCH;
 			}
 		}
-		else if ( E1/ E2 <= 1.0) {
+		else if ( snratio <= 1.0) {
 			waitcount = 8;
 			sigsearch = 0;
 		}
@@ -1715,9 +1796,6 @@ int psk::rx_process(const double *buf, int len)
 
 void psk::transmit(double *buf, int len)
 {
-	if (btn_imd_on->value())
-		for (int i = 0; i < len; i++) xmtfilt->Irun(buf[i], buf[i]);
-
 	ModulateXmtr(buf, len);
 }
 
@@ -1780,6 +1858,21 @@ void psk::tx_carriers()
 		for (int i = 0; i < symbollen; i++) {
 
 			shapeA = tx_shape[i];
+
+			if (test_signal_window && test_signal_window->visible() && btn_imd_on->value()) {
+				double imd = pow(10, xmtimd->value()/20.0);
+				shapeA -= (imd * imd_shape[i]);
+//				shapeA = cos(i * M_PI/symbollen);
+//				shapeA -= imd * cos(3.0 * i * M_PI / symbollen);
+//				shapeA -= imd * (3.0 / 5.0) * cos(5.0 * i * M_PI / symbollen);
+//				shapeA -= imd * (3.0 / 7.0) * cos(7.0 * i * M_PI / symbollen);
+//				shapeA -= imd * (3.0 / 9.0) * cos(9.0 * i * M_PI / symbollen);
+//				shapeA *= 0.5;
+//				shapeA += 0.5;
+			}
+//			else
+//				shapeA = tx_shape[i];  //0.5 * cos(i * M_PI / symbollen) + 0.5;
+
 			shapeB = (1.0 - shapeA);
 
 			ival = shapeA * prevsymbol[car].real() + shapeB * symbol.real();
@@ -1790,11 +1883,6 @@ void psk::tx_carriers()
 			} else {
 				outbuf[i] = (ival * cos(phaseacc[car]) + qval * sin(phaseacc[car])) / numcarriers;
 			}
-// create an imd value
-			double maxmag = xmtimd->value();
-			if (btn_imd_on->value())
-				if (fabs(outbuf[i]) > maxmag)
-					outbuf[i] = maxmag * (outbuf[i] < 0 ? -1 : 1);
 
 			phaseacc[car] += delta[car];
 			if (phaseacc[car] > TWOPI) phaseacc[car] -= TWOPI;
@@ -1817,7 +1905,6 @@ void psk::tx_carriers()
 	maxamp = 0;
 	for (int i = 0; i < symbollen; i++)
 		if (maxamp < fabs(outbuf[i])) maxamp = fabs(outbuf[i]);
-	maxamp *= 1.02;
 	if (maxamp) {
 		for (int i = 0; i < symbollen; i++)
 			outbuf[i] /= maxamp;
@@ -1883,80 +1970,22 @@ void psk::tx_xpsk(int bit)
 	int fecbits = 0;
 
 	// If invalid value of bitcount, reset to 0
-	if (_puncturing || _xpsk || _16psk)
+	if ( (_8psk && _puncturing) || _xpsk || _16psk)
 		if ( (bitcount & 0x1) )
 			bitcount = 0;
 
-	//printf("\n\n bit: %d", bit);
-
 	// Pass one bit and return two bits
 	bitshreg = enc->encode(bit);
 	// Interleave
 	Txinlv->bits(&bitshreg); // Bit-interleave
 	fecbits = bitshreg;
 
-
-	/// DEBUG
-	/*
-	 * 	static bool flip;
-	 if (flip) {
-	 flip = false;
-	 fecbits = 0;
-	 } else {
-	 flip = true;
-	 fecbits = 3;
-	 }
-	 */
-
-	//printf("\nfecbits: %u", fecbits);
-	//printf("\nbitcount: %d", bitcount);
-
 	if (_xpsk) { // 2 bits-per-symbol. Transmit every call
 		xpsk_sym = static_cast<unsigned int>(fecbits);
 		tx_symbol(xpsk_sym);
 		return;
 	}
 
-	// DEBUG
-	/*
-	 if (_8psk) {
-	 tx_symbol(0);
-	 tx_symbol(7);
-	 return;
-	 }
-	 */
-
-	// DEBUG, send a known pattern of symbols / bits
-	/*
-	 * 	static int counter = 0;
-	 counter++;
-	 if ( counter > 7 ) counter = 0;
-	 tx_symbol(1);
-	 return;
-	 */
-
-	/*
-	 else if (mode == MODE_8PSK ???) { // Puncture @ 5/6 rate | tx 3bits/symbol (8psk)
-	 // Collect up to 8 bits
-	 if ( x_bitcount < 8 ) {
-	 x_xpsk_sym |= (static_cast<unsigned int>(fecbits) & 1) << x_bitcount ;
-	 x_xpsk_sym |= (static_cast<unsigned int>(fecbits) & 2) << x_bitcount ;
-	 x_bitcount += 2;
-	 return;
-
-	 // When 10 bits are buffered,
-	 //	drop 4 bits then transmit 6 bits (2-symbols)
-	 } else {
-	 x_xpsk_sym |= (static_cast<unsigned int>(fecbits) & 1) << x_bitcount ;
-	 x_xpsk_sym |= (static_cast<unsigned int>(fecbits) & 2) << x_bitcount ;
-	 tx_symbol( (x_xpsk_sym & 14) >> 1);
-	 tx_symbol( (x_xpsk_sym & 448) >> 6);
-	 x_xpsk_sym = x_bitcount = 0;
-	 return;
-	 }
-	 }
-	 */
-
 	else if (_8psk && _puncturing) { // @ 2/3 rate
 
 		if ( 0 == bitcount) {
@@ -1966,8 +1995,8 @@ void psk::tx_xpsk(int bit)
 
 		} else if ( 2 == bitcount ) {
 			xpsk_sym |= (static_cast<unsigned int>(fecbits) & 1) << 2 ;
-			/// Puncture -> //xpsk_sym |= (static_cast<unsigned int>(fecbits) & 2) << 2 ;
-			tx_symbol(xpsk_sym & 7);
+			/// Punctured anyways, so skip -> //xpsk_sym |= (static_cast<unsigned int>(fecbits) & 2) << 2 ;
+			tx_symbol(xpsk_sym & 7); /// Drop/puncture the high-bit on Tx
 			xpsk_sym = bitcount = 0;
 			return;
 		}
@@ -1977,35 +2006,21 @@ void psk::tx_xpsk(int bit)
 	else if (_8psk) { // 3 bits-per-symbol. Accumulate then tx.
 
 		if ( 0 == bitcount ) { // Empty xpsk_sym buffer: add 2 bits and return
-							   //printf("\nxpsk_sym|preadd: %u", xpsk_sym);
 			xpsk_sym = static_cast<unsigned int>(fecbits);
-			//printf("\nxpsk_sym|postadd: %u", xpsk_sym);
 			bitcount = 2;
 			return ;
 
 		} else if ( 1 == bitcount ) { // xpsk_sym buffer with one bit: add 2 bits then tx and clear
-									  //xpsk_sym <<= 2; // shift left 2 bits
-
-			//printf("\nxpsk_sym|preadd: %u", xpsk_sym);
 			xpsk_sym |= (static_cast<unsigned int>(fecbits) & 1) << 1 ;
 			xpsk_sym |= (static_cast<unsigned int>(fecbits) & 2) << 1 ;
-			//printf("\nxpsk_sym|postadd: %u", xpsk_sym);
-
-			//printf("\nxpsk_sym|postinlv: %u", xpsk_sym);
 			tx_symbol(xpsk_sym);
 			xpsk_sym = bitcount = 0;
 			return;
 
 		} else if ( 2 == bitcount ) { // xpsk_sym buffer with 2 bits: add 1 then tx and save next bit
-									  //printf("\nxpsk_sym|preadd: %u", xpsk_sym);
 			xpsk_sym |= (static_cast<unsigned int>(fecbits) & 1) << 2 ;
-			//printf("\nxpsk_sym|postadd: %u", xpsk_sym);
-
-			//printf("\nxpsk_sym|postinlv: %u", xpsk_sym);
-
 			tx_symbol(xpsk_sym);
 			xpsk_sym = bitcount = 0;
-
 			xpsk_sym |= (static_cast<unsigned int>(fecbits) & 2) >> 1 ;
 			bitcount = 1;
 			return;
@@ -2092,16 +2107,19 @@ void psk::tx_flush()
 			tx_bit(1);
 			tx_bit(1);
 		}
-		// QPSK - flush the encoder
+
+	// QPSK - flush the encoder
 	} else if (_qpsk) {
 		for (int i = 0; i < dcdbits; i++)
 			tx_bit(0);
-		// FEC : replace unmodulated carrier by an encoded sequence of zeros
+
+	// FEC enabled: Sens the NULL character in MFSk varicode as the flush / post-amble sequence
 	} else if (!_disablefec && (_xpsk || _8psk || _16psk) ) {
-		for (int i = 0; i < flushlength; i++) {
-			tx_char(0);   // <NUL>
+		for (int i=0; i<flushlength; i++) {
+			tx_char(0); // Send <NUL> to clear bit accumulators on both Tx and Rx ends.
 		}
-		// FEC disabled: use unmodulated carrier
+
+	// FEC disabled: use unmodulated carrier (bpsk-like single tone)
 	} else if (_disablefec && ( _xpsk || _8psk || _16psk) ) {
 		for (int i=0; i<symbits; i++) {
 			tx_char(0); // Send <NUL> to clear bit accumulators on both Tx and Rx ends.
@@ -2110,20 +2128,18 @@ void psk::tx_flush()
 		int symbol;
 		if (_16psk) symbol = 8;
 		else if (_8psk) symbol = 4;
-		else symbol = 2;
+		else symbol = 2; // xpsk
 
-		int _dcdbits = dcdbits - 1;
-		if(progStatus.psk8DCDShortFlag)
-			_dcdbits  = 32/(symbits - 1);
+		for (int i = 0; i <= 96; i++) // DCD window is only 32-bits wide. Send 3-times
+			tx_symbol(symbol);
 
-		for (int i = 0; i <= _dcdbits; i++) // DCD window is only 32-bits wide
-			tx_symbol(symbol); // 0 degrees
-							   // Standard BPSK postamble
-							   // DCD off sequence (unmodulated carrier)
+	// Standard BPSK postamble
 	} else {
 		for (int i = 0; i < dcdbits; i++)
 			tx_symbol(2); // 0 degrees
 	}
+	for (int i = 0; i < 2048; i++) outbuf[i] = 0;
+	transmit(outbuf, 2048);
 }
 
 // Necessary to clear the interleaver before we start sending
@@ -2136,43 +2152,59 @@ void psk::clearbits()
 
 int psk::tx_process()
 {
-	int c;
+	modem::tx_process();
 
-	// DCD window is only 32 bits, send a maximum of 3-times.
-	if(progStatus.psk8DCDShortFlag) {
-		if ( (_8psk || _xpsk || _16psk) && preamble > 96)
-			preamble = 96;
-	}
-
-	if (preamble > 0) {
-		if (_pskr || ((_xpsk || _8psk || _16psk) && !_disablefec) ) {
-			if (startpreamble == true) {
-				if (mode != MODE_PSK63F) clearbits();
-				startpreamble = false;
-			}
+	if (preamble) {
+		if (_pskr) {
+			if (mode != MODE_PSK63F)
+				clearbits();
 			// FEC prep the encoder with one/zero sequences of bits
-			preamble--;
-			preamble--;
-			tx_bit(1);
-			tx_bit(0);
+			for (int i = 0; i < preamble; i += 2) {
+				tx_bit(1);
+				tx_bit(0);
+			}
 			// FEC: Mark start of first character with a double zero
 			// to ensure sync at end of preamble
-			if (preamble == 0) {
-				while (acc_symbols % numcarriers) tx_bit(0);
-				tx_char(0); // <NUL>
-			}
+			while (acc_symbols % numcarriers) tx_bit(0);
+			tx_char(0); // <NUL>
+			preamble = 0;
 			return 0;
+		} else if (_8psk) {
+			if (!_disablefec) clearbits();
+			if(progStatus.psk8DCDShortFlag)
+				if (preamble > 96) preamble = 96;
+			if (_disablefec) {
+				// Send continuous symbol 0: Usual PSK 2-tone preamble
+				for (int i = 0; i < preamble; i++ )
+					tx_symbol(0);
+				tx_char(0);
+				preamble = 0;
+				return 0;
+			} else {
+				_disablefec = true;
+				for (int i = 0; i < preamble/2; i++) tx_symbol(0);
+				_disablefec = false;
+				// FEC prep the encoder with encoded sequence of double-zeros
+				// sends a single centered preamble tone for FEC modes
+				for (int i = 0; i < preamble; i += 2) {
+					tx_bit(0);
+					tx_bit(0);
+				}
+				tx_char(0);
+				preamble = 0;
+				return 0;
+			}
 		} else {
-			//JD for QPSK500R
-			//			if (mode == MODE_QPSK500) clearbits();
 			// Standard BPSK/QPSK preamble
-			preamble--;
-			tx_symbol(0);   // send phase reversals
+			for (int i = 0; i < preamble; i++) {
+				tx_symbol(0);   // send phase reversals
+			}
+			preamble = 0;
 			return 0;
 		}
 	}
 
-	c = get_tx_char();
+	int c = get_tx_char();
 
 	if (c == GET_TX_CHAR_ETX || stopflag) {
 		tx_flush();
@@ -2210,58 +2242,54 @@ int psk::tx_process()
 // derived from pskcore by Moe Wheatley, AE4JY
 //============================================================================
 
+static bool reset_filters;
+
 void psk::initSN_IMD()
 {
-	for(int i = 0; i < NUM_FILTERS; i++)
-	{
-		I1[i] = I2[i] = Q1[i] = Q2[i] = 0.0;
-		m_Energy[i] = 0.0;
-	}
-	m_NCount = 0;
-
-	COEF[0] = 2.0 * cos(TWOPI * 9 / GOERTZEL);
-	COEF[1] = 2.0 * cos(TWOPI * 36 / GOERTZEL);
-	COEF[2] = 2.0 * cos(TWOPI  * 27 / GOERTZEL);
+	reset_filters = true;
 }
 
 void psk::resetSN_IMD()
 {
-	for(int i = 0; i < NUM_FILTERS; i++) {
-		I1[i] = I2[i] = Q1[i] = Q2[i] = 0.0;
-	}
-	m_NCount = 0;
+	reset_filters = true;
 }
 
 //============================================================================
 //  This routine calculates the energy in the frequency bands of
-//   carrier=F0(15.625), noise=F1(31.25), and
-//   3rd order product=F2(46.875)
+//   carrier = base (0), fundamental = F1(15.625), noise = F2(31.25), and
+//   3rd order product = F3(46.875)
 //  It is called with cmplx data samples at 500 Hz.
 //============================================================================
 
 void psk::calcSN_IMD(cmplx z)
 {
-	int i;
-	double tempI = 0, tempQ = 0;
+	if (!re_Gbin[0]) return;
 
-	for(i = 0; i < NUM_FILTERS; i++) {
-		tempI = I1[i];
-		tempQ = Q1[i];
-		I1[i] = I1[i] * COEF[i]- I2[i] + z.real();
-		Q1[i] = Q1[i] * COEF[i]- Q2[i] + z.imag();
-		I2[i] = tempI;
-		Q2[i] = tempQ;
-	}
+	if (reset_filters) {
+		e0_filt->reset();
+		e1_filt->reset();
+		e2_filt->reset();
+		e3_filt->reset();
 
-	if( ++m_NCount >= GOERTZEL ) {
-		m_NCount = 0;
-		for(i = 0; i < NUM_FILTERS; i++) {
-			m_Energy[i] =   I1[i]*I1[i] + Q1[i]*Q1[i]
-			+ I2[i]*I2[i] + Q2[i]*Q2[i]
-			- I1[i]*I2[i]*COEF[i]
-			- Q1[i]*Q2[i]*COEF[i];
-			I1[i] = I2[i] = Q1[i] = Q2[i] = 0.0;
+		for(int i = 0; i < NUM_FILTERS; i++) {
+			re_Gbin[i]->reset();
+			im_Gbin[i]->reset();
 		}
-		signalquality();
+		reset_filters = false;
 	}
+
+	bool isvalid = true;
+
+	for (int i = 0; i < NUM_FILTERS; i++) {
+		isvalid &= re_Gbin[i]->run(real(z));
+		isvalid &= im_Gbin[i]->run(imag(z));
+	}
+	if (isvalid) {
+		signalquality();
+		for (int i = 0; i < NUM_FILTERS; i++) {
+			re_Gbin[i]->reset();
+			im_Gbin[i]->reset();
+		}
+	}
+	return;
 }
