@@ -44,9 +44,8 @@
 
 char throbmsg[80];
 
-void  throb::tx_init(SoundBase *sc)
+void  throb::tx_init()
 {
-	scard = sc;
 	preamble = 4;
 	reset_syms();
 	videoText();
@@ -69,6 +68,16 @@ void throb::init()
 	modem::init();
 	rx_init();
 	set_scope_mode(Digiscope::SCOPE);
+
+	if (progdefaults.StartAtSweetSpot)
+		set_freq(progdefaults.PSKsweetspot);
+	else if (progStatus.carrier != 0) {
+		set_freq(progStatus.carrier);
+#if !BENCHMARK_MODE
+		progStatus.carrier = 0;
+#endif
+	} else
+		set_freq(wf->Carrier());
 }
 
 throb::~throb()
@@ -609,6 +618,8 @@ void throb::send(int symbol)
 
 int throb::tx_process()
 {
+	modem::tx_process();
+
 	int i, c, sym;
 
 	if (preamble > 0) {
