@@ -38,6 +38,9 @@
 #include "qrunner.h"
 #include "macros.h"
 
+#include "testsigs.h"
+#include "test_signal.h"
+
 #include "status.h"
 #include "debug.h"
 
@@ -199,7 +202,6 @@ modem *throbx4_modem = 0;
 
 modem *wwv_modem = 0;
 modem *anal_modem = 0;
-modem *fftscan_modem = 0;
 modem *ssb_modem = 0;
 
 double modem::frequency = 1000;
@@ -215,10 +217,10 @@ modem::modem()
 {
 	scptr = 0;
 
-	if( !progdefaults.retain_freq_lock ) {
-		freqlock = false;
+	if (wf)
+		frequency = tx_frequency = wf->Carrier();
+	else
 		frequency = tx_frequency = 1000;
-	}
 
 	sigsearch = 0;
 	if (wf) {
@@ -245,16 +247,6 @@ void modem::init()
 	bool wfrev = wf->Reverse();
 	bool wfsb = wf->USB();
 	reverse = wfrev ^ !wfsb;
-
-	if (progdefaults.StartAtSweetSpot) {
-		set_freq(progdefaults.PSKsweetspot);
-	} else if (progStatus.carrier != 0) {
-		set_freq(progStatus.carrier);
-#if !BENCHMARK_MODE
-		progStatus.carrier = 0;
-#endif
-	} else
-		set_freq(wf->Carrier());
 }
 
 void modem::set_freq(double freq)
@@ -290,6 +282,10 @@ double modem::get_txfreq_woffset(void) const
 	if (mailserver && progdefaults.PSKmailSweetSpot)
 		return (progdefaults.PSKsweetspot - progdefaults.TxOffset);
 	if (get_mode() == MODE_FSQ) return (1500 - progdefaults.TxOffset);
+
+	if (test_signal_window && test_signal_window->visible() && btnOffsetOn->value())
+		return tx_frequency + ctrl_freq_offset->value();
+
 	return (tx_frequency - progdefaults.TxOffset);
 }
 
@@ -438,16 +434,12 @@ double modem::gauss(double sigma) {
 // simulating the noise that is added to the signal.
 // return signal + noise, limiting value to +/- 1.0
 
-//static double mag;
-
 void modem::add_noise(double *buffer, int len)
 {
 	double sigma = sigmaN(noiseDB->value());
 	double sn = 0;
+
 	for (int n = 0; n < len; n++) {
-//		mag = fabs(buffer[n]);
-//		if (btn_imd_on->value())
-//			buffer[n] *= (1.0 - mag *xmtimd->value())/(1.0 - xmtimd->value());
 		if (btnNoiseOn->value()) {
 			sn = (buffer[n] + gauss(sigma)) / (1.0 + 3.0 * sigma);
 			buffer[n] = clamp(sn, -1.0, 1.0);
@@ -464,10 +456,11 @@ void modem::s2nreport(void)
 }
 
 bool disable_modem = false;
+#define SIGLIMIT 0.95
 
 void modem::ModulateXmtr(double *buffer, int len)
 {
-	if (unlikely(!scard)) return;
+	if (unlikely(!TXscard)) return;
 	if (disable_modem) return;
 
 	tx_sample_count += len;
@@ -479,23 +472,23 @@ void modem::ModulateXmtr(double *buffer, int len)
 		return;
 	}
 
-	if (withnoise) add_noise(buffer, len);
+	if (test_signal_window && test_signal_window->visible()) add_noise(buffer, len);
 
 	if (progdefaults.viewXmtSignal &&
 		!(PERFORM_CPS_TEST || active_modem->XMLRPC_CPS_TEST))
 		trx_xmit_wfall_queue(samplerate, buffer, (size_t)len);
 
 	double mult = pow(10, progdefaults.txlevel / 20.0);
-	if (mult > 0.99) mult = 0.99;
+	if (mult > SIGLIMIT) mult = SIGLIMIT;
 	for (int i = 0; i < len; i++) {
 		buffer[i] *= mult;
-		if (buffer[i] < -0.99) buffer[i] = -0.99;
-		if (buffer[i] >  0.99) buffer[i] = 0.99;
+		if (buffer[i] < -SIGLIMIT) buffer[i] = -SIGLIMIT;
+		if (buffer[i] >  SIGLIMIT) buffer[i] = SIGLIMIT;
 	}
 
 	try {
 		unsigned n = 4;
-		while (scard->Write(buffer, len) == 0 && --n);
+		while (TXscard->Write(buffer, len) == 0 && --n);
 		if (n == 0)
 			throw SndException(-99, "Sound write failed");
 	}
@@ -510,33 +503,34 @@ void modem::ModulateXmtr(double *buffer, int len)
 
 #include <iostream>
 using namespace std;
+
 void modem::ModulateStereo(double *left, double *right, int len, bool sample_flag)
 {
-	if (unlikely(!scard)) return;
+	if (unlikely(!TXscard)) return;
 	if (disable_modem) return;
 
 	if(sample_flag)
 		tx_sample_count += len;
 
-	if (withnoise && progdefaults.noise) add_noise(left, len);
+	if (test_signal_window && test_signal_window->visible() && progdefaults.noise) add_noise(left, len);
 
 	if (progdefaults.viewXmtSignal &&
 		!(PERFORM_CPS_TEST || active_modem->XMLRPC_CPS_TEST))
 		trx_xmit_wfall_queue(samplerate, left, (size_t)len);
 
 	double mult = pow(10, progdefaults.txlevel / 20.0);
-	if (mult > 0.99) mult = 0.99;
+	if (mult > SIGLIMIT) mult = SIGLIMIT;
 
 	for (int i = 0; i < len; i++) {
-		if (right[i] < -0.99) right[i] = -0.99;
-		if (right[i] >  0.99) right[i] = 0.99;
+		if (right[i] < -SIGLIMIT) right[i] = -SIGLIMIT;
+		if (right[i] >  SIGLIMIT) right[i] = SIGLIMIT;
 		left[i] *= mult;
-		if (left[i] < -0.99) left[i] = -0.99;
-		if (left[i] >  0.99) left[i] = 0.99;
+		if (left[i] < -SIGLIMIT) left[i] = -SIGLIMIT;
+		if (left[i] >  SIGLIMIT) left[i] = SIGLIMIT;
 	}
 	try {
 		unsigned n = 4;
-		while (scard->Write_stereo(left, right, len) == 0 && --n);
+		while (TXscard->Write_stereo(left, right, len) == 0 && --n);
 		if (n == 0)
 			throw SndException(-99, "Sound write failed");
 	}
@@ -555,29 +549,29 @@ void modem::ModulateStereo(double *left, double *right, int len, bool sample_fla
 //------------------------------------------------------------------------------
 void modem::ModulateVideoStereo(double *left, double *right, int len, bool sample_flag)
 {
-	if (unlikely(!scard)) return;
+	if (unlikely(!TXscard)) return;
 
 	if(sample_flag)
 		tx_sample_count += len;
 
-	if (withnoise && progdefaults.noise) add_noise(left, len);
+	if (test_signal_window && test_signal_window->visible() && progdefaults.noise) add_noise(left, len);
 
 	if (progdefaults.viewXmtSignal &&
 		!(PERFORM_CPS_TEST || active_modem->XMLRPC_CPS_TEST))
 		trx_xmit_wfall_queue(samplerate, left, (size_t)len);
 
-	double mult = 0.99 * pow(10, progdefaults.txlevel / 20.0);
+	double mult = SIGLIMIT * pow(10, progdefaults.txlevel / 20.0);
 
 	for (int i = 0; i < len; i++) {
-		if (right[i] < -1.0) right[i] = -1.0;
-		if (right[i] >  1.0) right[i] = 1.0;
+		if (right[i] < -SIGLIMIT) right[i] = -SIGLIMIT;
+		if (right[i] >  SIGLIMIT) right[i] = SIGLIMIT;
 		left[i] *= mult;
-		if (left[i] < -1.0) left[i] = -1.0;
-		if (left[i] >  1.0) left[i] = 1.0;
+		if (left[i] < -SIGLIMIT) left[i] = -SIGLIMIT;
+		if (left[i] >  SIGLIMIT) left[i] = SIGLIMIT;
 	}
 	try {
 		unsigned n = 4;
-		while (scard->Write_stereo(left, right, len) == 0 && --n);
+		while (TXscard->Write_stereo(left, right, len) == 0 && --n);
 		if (n == 0)
 			throw SndException(-99, "Sound write failed");
 	}
@@ -592,7 +586,7 @@ void modem::ModulateVideoStereo(double *left, double *right, int len, bool sampl
 
 void modem::ModulateVideo(double *buffer, int len)
 {
-	if (unlikely(!scard)) return;
+	if (unlikely(!TXscard)) return;
 
 	tx_sample_count += len;
 
@@ -603,22 +597,22 @@ void modem::ModulateVideo(double *buffer, int len)
 		return;
 	}
 
-	if (withnoise && progdefaults.noise) add_noise(buffer, len);
+	if (test_signal_window && test_signal_window->visible() && progdefaults.noise) add_noise(buffer, len);
 
 	if (progdefaults.viewXmtSignal &&
 		!(PERFORM_CPS_TEST || active_modem->XMLRPC_CPS_TEST))
 		trx_xmit_wfall_queue(samplerate, buffer, (size_t)len);
 
-	double mult = 0.99 * pow(10, progdefaults.txlevel / 20.0);
+	double mult = SIGLIMIT * pow(10, progdefaults.txlevel / 20.0);
 	for (int i = 0; i < len; i++) {
 		buffer[i] *= mult;
-		if (buffer[i] < -1.0) buffer[i] = -1.0;
-		if (buffer[i] >  1.0) buffer[i] = 1.0;
+		if (buffer[i] < -SIGLIMIT) buffer[i] = -SIGLIMIT;
+		if (buffer[i] >  SIGLIMIT) buffer[i] = SIGLIMIT;
 	}
 
 	try {
 		unsigned n = 4;
-		while (scard->Write(buffer, len) == 0 && --n);
+		while (TXscard->Write(buffer, len) == 0 && --n);
 		if (n == 0)
 			throw SndException(-99, "Sound write failed");
 	}
@@ -725,7 +719,7 @@ void modem::cwid_send_symbol(int bits)
 		sample = 0,
 		currsym = bits & 1;
 
-	freq = get_txfreq() - progdefaults.TxOffset;
+	freq = get_txfreq_woffset() - progdefaults.TxOffset;
 
 	if ((currsym == 1) && (cwid_lastsym == 0))
 		cwid_phaseacc = 0.0;
@@ -769,7 +763,7 @@ void modem::cwid_send_symbol(int bits)
 
 void modem::cwid_send_ch(int ch)
 {
-	int code;
+	std::string code;
 
 // handle word space separately (7 dots spacing)
 // last char already had 2 elements of inter-character spacing
@@ -785,17 +779,20 @@ void modem::cwid_send_ch(int ch)
 	}
 
 // convert character code to a morse representation
-	if ((ch < 256) && (ch >= 0)) {
-		code = morse.tx_lookup(ch); //cw_tx_lookup(ch);
-	} else {
-		code = 0x04; 	// two extra dot spaces
-	}
-
+	code = morse.tx_lookup(ch); //cw_tx_lookup(ch);
+	if (!code.length())
+		return;
 // loop sending out binary bits of cw character
-	while (code > 1) {
-		cwid_send_symbol(code);// & 1);
-		code = code >> 1;
+	for (size_t n = 0; n < code.length(); n++) {
+		cwid_send_symbol(0);
+		cwid_send_symbol(1);
+		if (code[n] == '-') {
+			cwid_send_symbol(1);
+			cwid_send_symbol(1);
+		}
 	}
+	cwid_send_symbol(0);
+	cwid_send_symbol(0);
 
 }
 
@@ -850,7 +847,7 @@ void modem::wfid_make_tones(int numchars)
 {
 	double f, flo, fhi;
 	int vwidth = (numchars*NUMCOLS + (numchars-1)*CHARSPACE - 1);
-	f = get_txfreq() + TONESPACING * vwidth/2.0;
+	f = get_txfreq_woffset() + TONESPACING * vwidth/2.0;
 	fhi = f + TONESPACING;
 	flo = fhi - (vwidth + 2) * TONESPACING;
 	for (int i = 1; i <= NUMCOLS * numchars; i++) {
@@ -921,7 +918,7 @@ void modem::pretone()
 {
 	int sr = get_samplerate();
 	int symlen = sr / 10;
-	double phaseincr = 2.0 * M_PI * get_txfreq() / sr;
+	double phaseincr = 2.0 * M_PI * get_txfreq_woffset() / sr;
 	double phase = 0.0;
 	double outbuf[symlen];
 
