@@ -30,6 +30,7 @@
 #include <semaphore.h>
 #include <cstdlib>
 #include <string>
+#include <string.h>
 
 #include "trx.h"
 #include "main.h"
@@ -49,6 +50,14 @@
 #include "rigsupport.h"
 #include "psm/psm.h"
 
+#include "icons.h"
+
+#include "fft-monitor.h"
+
+extern fftmon *fft_modem;
+
+#include "spectrum_viewer.h"
+
 #if BENCHMARK_MODE
 #  include "benchmark.h"
 #endif
@@ -75,7 +84,13 @@ state_t 	trx_state;
 modem		*active_modem = 0;
 cRsId		*ReedSolomon = 0;
 cDTMF		*dtmf = 0;
-SoundBase 	*scard;
+SoundBase 	*RXscard = 0;
+bool		RXsc_is_open = false;
+bool		TXsc_is_open = false;
+SoundBase	*TXscard = 0;
+static int	current_RXsamplerate = 0;
+static int	current_TXsamplerate = 0;
+
 static int	_trx_tune;
 
 // Ringbuffer for the audio "history". A pointer into this buffer
@@ -106,11 +121,12 @@ static void trx_xmit_wfall_draw(int samplerate)
 	rv[1].buf = 0;
 
 #define block_read_(vec_)						\
-	while (vec_.len >= WFBLOCKSIZE) {				\
-		wf->sig_data(vec_.buf, WFBLOCKSIZE, samplerate);	\
+	while (vec_.len >= WFBLOCKSIZE) {			\
+		wf->sig_data(vec_.buf, samplerate);		\
+		REQ(&waterfall::handle_sig_data, wf);	\
 		vec_.len -= WFBLOCKSIZE;				\
 		vec_.buf += WFBLOCKSIZE;				\
-		trxrb.read_advance(WFBLOCKSIZE);				\
+		trxrb.read_advance(WFBLOCKSIZE);		\
 	}
 
 	trxrb.get_rv(rv);
@@ -128,7 +144,8 @@ static void trx_xmit_wfall_draw(int samplerate)
 		double buf[WFBLOCKSIZE];
 		do {
 			trxrb.read(buf, WFBLOCKSIZE);
-			wf->sig_data(buf, WFBLOCKSIZE, samplerate);
+			wf->sig_data(buf, samplerate);
+			REQ(&waterfall::handle_sig_data, wf);
 		} while (trxrb.read_space() >= WFBLOCKSIZE);
 	}
 }
@@ -160,7 +177,7 @@ static void trx_xmit_wfall_end(int samplerate)
 
 	trxrb.write_advance(pad);
 
-	REQ(trx_xmit_wfall_draw, samplerate);
+	trx_xmit_wfall_draw(samplerate);
 }
 
 // Copy buf to the ringbuffer if it has enough space. Queue a waterfall
@@ -190,7 +207,7 @@ void trx_xmit_wfall_queue(int samplerate, const double* buf, size_t len)
 
 	trxrb.write_advance(len);
 	if (trxrb.read_space() >= WFBLOCKSIZE)
-		REQ(trx_xmit_wfall_draw, samplerate);
+		trx_xmit_wfall_draw(samplerate);
 }
 
 //=============================================================================
@@ -198,7 +215,6 @@ void trx_xmit_wfall_queue(int samplerate, const double* buf, size_t len)
 void trx_trx_receive_loop()
 {
 	size_t  numread;
-	int  current_samplerate;
 	assert(powerof2(SCBLOCKSIZE));
 
 	if (unlikely(!active_modem)) {
@@ -212,20 +228,29 @@ void trx_trx_receive_loop()
 	return;
 #endif
 
-	if (unlikely(!scard)) {
-	MilliSleep(10);
-	return;
+	if (!RXscard) {
+		MilliSleep(10);
+		return;
 	}
 
 	try {
-		current_samplerate = active_modem->get_samplerate();
-		if (scard->Open(O_RDONLY, current_samplerate))
-			REQ(sound_update, progdefaults.btnAudioIOis);
+		if (!progdefaults.is_full_duplex || !RXsc_is_open ||
+			current_RXsamplerate != active_modem->get_samplerate() ) {
+			current_RXsamplerate = active_modem->get_samplerate();
+			if (RXscard) {
+				RXscard->Close(O_RDONLY);
+				RXscard->Open(O_RDONLY, current_RXsamplerate);
+				REQ(sound_update, progdefaults.btnAudioIOis);
+				RXsc_is_open = true;
+			}
+		}
 	}
 	catch (const SndException& e) {
 		LOG_ERROR("%s. line: %i", e.what(), __LINE__);
 		put_status(e.what(), 5);
-		scard->Close();
+		if (RXscard) RXscard->Close();
+		RXsc_is_open = false;
+		current_RXsamplerate = 0;
 //	if (e.error() == EBUSY && progdefaults.btnAudioIOis == SND_IDX_PORT) {
 		if (progdefaults.btnAudioIOis == SND_IDX_PORT) {
 			sound_close();
@@ -240,11 +265,15 @@ void trx_trx_receive_loop()
 	ringbuffer<double>::vector_type rbvec[2];
 	rbvec[0].buf = rbvec[1].buf = 0;
 
+	if (RXscard) RXscard->flush(O_RDONLY);
+
 	while (1) {
 		try {
 			numread = 0;
-			while (numread < SCBLOCKSIZE && trx_state == STATE_RX)
-				numread += scard->Read(fbuf + numread, SCBLOCKSIZE - numread);
+			if (RXscard) {
+				while (numread < SCBLOCKSIZE && trx_state == STATE_RX)
+					numread += RXscard->Read(fbuf + numread, SCBLOCKSIZE - numread);
+			}
 			if (numread > SCBLOCKSIZE) {
 				LOG_ERROR("numread error %lu", (unsigned long) numread);
 				numread = SCBLOCKSIZE;
@@ -253,7 +282,7 @@ void trx_trx_receive_loop()
 				for (size_t i = 0; i < numread; i++)
 					hsbuff[i] = fbuf[i];
 			} else {
-				if (trxrb.write_space() == 0) // discard some old data
+				if (trxrb.write_space() == 0) // diRXscard some old data
 					trxrb.read_advance(SCBLOCKSIZE);
 
 				size_t room = trxrb.get_wv(rbvec, numread);
@@ -268,7 +297,8 @@ void trx_trx_receive_loop()
 			}
 		}
 		catch (const SndException& e) {
-			scard->Close();
+			if (RXscard) RXscard->Close();
+			RXsc_is_open = false;
 			LOG_ERROR("%s. line: %i", e.what(), __LINE__);
 			put_status(e.what(), 5);
 			MilliSleep(10);
@@ -293,9 +323,12 @@ void trx_trx_receive_loop()
 			active_modem->HistoryON(false);
 		} else {
 			trxrb.write_advance(numread);
-			REQ(&waterfall::sig_data, wf, rbvec[0].buf, numread, current_samplerate);
-
+			wf->sig_data(rbvec[0].buf, current_RXsamplerate);
+			REQ(&waterfall::handle_sig_data, wf);
 			if (!bHistory) {
+				if (fft_modem && spectrum_viewer->visible())
+					fft_modem->rx_process(rbvec[0].buf, numread);
+
 				active_modem->rx_process(rbvec[0].buf, numread);
 				if (progdefaults.rsid)
 					ReedSolomon->receive(fbuf, numread);
@@ -317,8 +350,13 @@ void trx_trx_receive_loop()
 			}
 		}
 	}
-	if (scard->must_close(O_RDONLY))
-		scard->Close(O_RDONLY);
+	if (trx_state == STATE_RESTART)
+		return;
+
+	if (!progdefaults.is_full_duplex ) {
+		if (RXscard) RXscard->Close(O_RDONLY);
+		RXsc_is_open = false;
+	}
 }
 
 
@@ -327,20 +365,25 @@ void trx_trx_transmit_loop()
 {
 	if (rx_only) return;
 
-	int  current_samplerate;
-	if (!scard) {
+	if (!TXscard) {
 		MilliSleep(10);
 		return;
 	}
 	if (active_modem) {
 
 		try {
-			current_samplerate = active_modem->get_samplerate();
-			scard->Open(O_WRONLY, current_samplerate);
-		}
-		catch (const SndException& e) {
+			if (current_TXsamplerate != active_modem->get_samplerate() || !TXsc_is_open) {
+				current_TXsamplerate = active_modem->get_samplerate();
+				if (TXscard) {
+					TXscard->Close(O_WRONLY);
+					TXscard->Open(O_WRONLY, current_TXsamplerate);
+					TXsc_is_open = true;
+				}
+			}
+		} catch (const SndException& e) {
 			LOG_ERROR("%s. line: %i", e.what(), __LINE__);
 			put_status(e.what(), 1);
+			current_TXsamplerate = 0;
 			MilliSleep(10);
 			return;
 		}
@@ -352,7 +395,7 @@ void trx_trx_transmit_loop()
 			push2talk->set(true);
 			REQ(&waterfall::set_XmtRcvBtn, wf, true);
 		}
-		active_modem->tx_init(scard);
+		active_modem->tx_init();
 
 		if ( ReedSolomon->assigned(active_modem->get_mode()) &&
 			 (progdefaults.TransmitRSid || progStatus.n_rsids != 0)) {
@@ -386,15 +429,17 @@ void trx_trx_transmit_loop()
 							trx_state = STATE_RX;
 				}
 				catch (const SndException& e) {
-					scard->Close();
+					if (TXscard) TXscard->Close();
+					TXsc_is_open = false;
 					LOG_ERROR("%s", e.what());
 					put_status(e.what(), 5);
+					current_TXsamplerate = 0;
 					MilliSleep(10);
 					return;
 				}
 			}
 		} else
-			if (trx_state != STATE_ABORT)
+			if (trx_state != STATE_ABORT && trx_state != STATE_RESTART)
 				trx_state = STATE_RX;
 
 		if (ReedSolomon->assigned(active_modem->get_mode()) &&
@@ -404,11 +449,16 @@ void trx_trx_transmit_loop()
 
 		progStatus.n_rsids = 0;
 
-		trx_xmit_wfall_end(current_samplerate);
+		trx_xmit_wfall_end(current_TXsamplerate);
 
-		scard->flush();
-		if (scard->must_close(O_WRONLY))
-			scard->Close(O_WRONLY);
+		if (TXscard) TXscard->flush();
+
+		if (trx_state == STATE_RX) {
+			if (!progdefaults.is_full_duplex) {
+				if (TXscard) TXscard->Close(O_WRONLY);
+				TXsc_is_open = false;
+			}
+		}
 
 	} else
 		MilliSleep(10);
@@ -426,47 +476,72 @@ void trx_tune_loop()
 {
 	if (rx_only) return;
 
-	int  current_samplerate;
-	if (!scard) {
+	if (!TXscard) {
 		MilliSleep(10);
 		return;
 	}
 	if (active_modem) {
 		try {
-			current_samplerate = active_modem->get_samplerate();
-			scard->Open(O_WRONLY, current_samplerate);
-		}
-		catch (const SndException& e) {
+			if (!progdefaults.is_full_duplex || !TXsc_is_open ||
+				current_TXsamplerate != active_modem->get_samplerate() ) {
+				current_TXsamplerate = active_modem->get_samplerate();
+				if (TXscard) TXscard->Close(O_WRONLY);
+				if (TXscard) {
+					TXscard->Open(O_WRONLY, current_TXsamplerate);
+					TXsc_is_open = true;
+				}
+			}
+		} catch (const SndException& e) {
 			LOG_ERROR("%s. line: %i", e.what(), __LINE__);
 			put_status(e.what(), 1);
 			MilliSleep(10);
+			current_TXsamplerate = 0;
 			return;
 		}
 
 		push2talk->set(true);
-		active_modem->tx_init(scard);
+		active_modem->tx_init();
 
 		try {
-			while (trx_state == STATE_TUNE) {
-				if (_trx_tune == 0) {
-					REQ(&waterfall::set_XmtRcvBtn, wf, true);
-					xmttune::keydown(active_modem->get_txfreq_woffset(), scard);
-					_trx_tune = 1;
-				} else
-					xmttune::tune(active_modem->get_txfreq_woffset(), scard);
+			if ((active_modem->get_mode() == MODE_CW)  && (use_nanoIO || progStatus.WK_online)) {
+				if (use_nanoIO) nanoCW_tune(1);
+				else WK_tune(1);
+
+				REQ(&waterfall::set_XmtRcvBtn, wf, true);
+				while (trx_state == STATE_TUNE) MilliSleep(10);
+
+				if (use_nanoIO) nanoCW_tune(0);
+				else WK_tune(0);
+			}
+			else {
+				while (trx_state == STATE_TUNE) {
+					if (_trx_tune == 0) {
+						REQ(&waterfall::set_XmtRcvBtn, wf, true);
+						xmttune::keydown(active_modem->get_txfreq_woffset(), TXscard);
+						_trx_tune = 1;
+					} else
+						xmttune::tune(active_modem->get_txfreq_woffset(), TXscard);
+				}
+				xmttune::keyup(active_modem->get_txfreq_woffset(), TXscard);
 			}
-			xmttune::keyup(active_modem->get_txfreq_woffset(), scard);
 		}
 		catch (const SndException& e) {
-			scard->Close();
+			if (TXscard) TXscard->Close();
+			TXsc_is_open = false;
 			LOG_ERROR("%s. line: %i", e.what(), __LINE__);
 			put_status(e.what(), 5);
 			MilliSleep(10);
+			current_TXsamplerate = 0;
 			return;
 		}
-		scard->flush();
-		if (scard->must_close(O_WRONLY))
-			scard->Close(O_WRONLY);
+		if (TXscard) TXscard->flush();
+
+		if (trx_state == STATE_RX) {
+			if (!progdefaults.is_full_duplex) {
+				if (TXscard) TXscard->Close(O_WRONLY);
+				TXsc_is_open = false;
+			}
+		}
 
 		_trx_tune = 0;
 	} else
@@ -503,15 +578,19 @@ void *trx_loop(void *args)
 
 		switch (trx_state) {
 		case STATE_ABORT:
-			delete scard;
-			scard = 0;
+			delete RXscard;
+			RXscard = 0;
+			delete TXscard;
+			TXscard = 0;
 			trx_state = STATE_ENDED;
 			// fall through
 		case STATE_ENDED:
 			REQ(set_flrig_ptt, 0);
+			stop_deadman();
 			return 0;
 		case STATE_RESTART:
 			REQ(set_flrig_ptt, 0);
+			stop_deadman();
 			trx_reset_loop();
 			break;
 		case STATE_NEW_MODEM:
@@ -520,14 +599,17 @@ void *trx_loop(void *args)
 			break;
 		case STATE_TX:
 			REQ(set_flrig_ptt, 1);
+			start_deadman();
 			trx_trx_transmit_loop();
 			break;
 		case STATE_TUNE:
 			REQ(set_flrig_ptt, 1);
+			start_deadman();
 			trx_tune_loop();
 			break;
 		case STATE_RX:
 			REQ(set_flrig_ptt, 0);
+			stop_deadman();
 			trx_trx_receive_loop();
 			break;
 		default:
@@ -544,7 +626,7 @@ static int new_freq;
 void trx_start_modem_loop()
 {
 	if (new_modem == active_modem) {
-		if (new_freq > 0)
+		if (new_freq > 0 && !progdefaults.retain_freq_lock)
 			active_modem->set_freq(new_freq);
 		active_modem->restart();
 		trx_state = STATE_RX;
@@ -559,7 +641,7 @@ void trx_start_modem_loop()
 
 	new_modem->init();
 	active_modem = new_modem;
-	if (new_freq > 0)
+	if (new_freq > 0 && !progdefaults.retain_freq_lock)
 		active_modem->set_freq(new_freq);
 	trx_state = STATE_RX;
 	REQ(&waterfall::opmode, wf);
@@ -580,31 +662,141 @@ void trx_start_modem(modem* m, int f)
 }
 
 //=============================================================================
+static string reset_loop_msg;
+void show_reset_loop_alert()
+{
+	progdefaults.btnAudioIOis = SND_IDX_NULL; // file i/o
+	sound_update(progdefaults.btnAudioIOis);
+	if (btnAudioIO[0]) {
+		fl_alert2("%s", reset_loop_msg.c_str());
+		btnAudioIO[0]->value(0);
+		btnAudioIO[1]->value(0);
+		btnAudioIO[2]->value(0);
+		btnAudioIO[3]->value(1);
+	}
+}
+
 void trx_reset_loop()
 {
-	if (scard)  {
-		delete scard;
-		scard = 0;
+	if (RXscard)  {
+		RXscard->Close();
+		RXsc_is_open = false;
+		delete RXscard;
+		RXscard = 0;
+	}
+	if (TXscard)  {
+		TXscard->Close();
+		TXsc_is_open = false;
+		delete TXscard;
+		TXscard = 0;
 	}
 
 	switch (progdefaults.btnAudioIOis) {
 #if USE_OSS
 	case SND_IDX_OSS:
-		scard = new SoundOSS(scDevice[0].c_str());
+		try {
+			RXscard = new SoundOSS(scDevice[0].c_str());
+			if (!RXscard) break;
+
+			RXscard->Open(O_RDONLY, current_RXsamplerate = 8000);
+			RXsc_is_open = true;
+
+			TXscard = new SoundOSS(scDevice[0].c_str());
+			if (!TXscard) break;
+
+			TXscard->Open(O_WRONLY, current_TXsamplerate = 8000);
+			TXsc_is_open = true;
+		} catch (...) {
+			reset_loop_msg = "OSS open failure";
+			REQ(show_reset_loop_alert);
+		}
 		break;
 #endif
 #if USE_PORTAUDIO
+/// All of this very convoluted logic is needed to allow a Linux user
+/// to switch from PulseAudio to PortAudio.  PulseAudio does not immediately
+/// release the sound card resources after closing the pulse audio object.
 	case SND_IDX_PORT:
-		scard = new SoundPort(scDevice[0].c_str(), scDevice[1].c_str());
+	{
+		RXscard = new SoundPort(scDevice[0].c_str(), scDevice[1].c_str());
+		TXscard = new SoundPort(scDevice[0].c_str(), scDevice[1].c_str());
+		unsigned long tm1 = zmsec();
+		int RXret = 0, TXret = 0;
+		int i;
+		RXsc_is_open = false;
+		TXsc_is_open = false;
+		for (i = 0; i < 10; i++) { // try 10 times
+			try {
+				if (!RXret)
+					RXret = RXscard->Open(O_RDONLY, current_RXsamplerate = 8000);
+				if (progdefaults.is_full_duplex) {
+					if (!TXret)
+						TXret = TXscard->Open(O_WRONLY, current_TXsamplerate = 8000);
+				}
+				if (RXret) RXsc_is_open = true;
+				if (TXret) TXsc_is_open = true;
+				break;
+			} catch (const SndException& e) {
+				MilliSleep(50);
+				Fl::awake();
+			}
+		}
+		unsigned long tm = zmsec() - tm1;
+		if (i == 10) {
+			if (RXscard) delete RXscard;
+			if (TXscard) delete TXscard;
+			RXscard = 0;
+			TXscard = 0;
+			LOG_PERROR("Port Audio device not available");
+			reset_loop_msg = "Port Audio device not available";
+			REQ(show_reset_loop_alert);
+		} else {
+			LOG_INFO ("Port Audio device available after %0.1f seconds", tm / 1000.0 );
+		}
 		break;
+	}
 #endif
 #if USE_PULSEAUDIO
 	case SND_IDX_PULSE:
-		scard = new SoundPulse(scDevice[0].c_str());
+		try {
+			RXscard = new SoundPulse(scDevice[0].c_str());
+			if (!RXscard) break;
+
+			RXscard->Open(O_RDONLY, current_RXsamplerate = 8000);
+			RXsc_is_open = true;
+
+			TXscard = new SoundPulse(scDevice[0].c_str());
+			if (!TXscard) break;
+		
+// needed to open playback device in PaVolumeControl
+			TXscard->Open(O_WRONLY, current_TXsamplerate = 8000);
+			double buffer[1024];
+			for (int i = 0; i < 1024; buffer[i++] = 0);
+			TXscard->Write_stereo(buffer, buffer, 1024);
+
+			if (progdefaults.is_full_duplex)
+				TXsc_is_open = true;
+			else {
+				TXscard->Close();
+				TXsc_is_open = false;
+			}
+		} catch (const SndException& e) {
+			LOG_ERROR("%s", e.what());
+			if (RXscard) delete RXscard;
+			if (TXscard) delete TXscard;
+			RXscard = 0;
+			TXscard = 0;
+			reset_loop_msg = "Pulse Audio error:\n";
+			reset_loop_msg.append(e.what());
+			reset_loop_msg.append("\n\nIs the server running?\nClose fldigi and execute 'pulseaudio --start'");
+			REQ(show_reset_loop_alert);
+		}
 		break;
 #endif
 	case SND_IDX_NULL:
-		scard = new SoundNull;
+		RXscard = new SoundNull;
+		TXscard = new SoundNull;
+		current_RXsamplerate = current_TXsamplerate = 0;
 		break;
 	default:
 		abort();
@@ -630,7 +822,14 @@ void trx_start(void)
 		return;
 	}
 
-	if (scard) delete scard;
+	if (RXscard) {
+		delete RXscard;
+		RXscard = 0;
+	}
+	if (TXscard) {
+		delete TXscard;
+		TXscard = 0;
+	}
 	if (ReedSolomon) delete ReedSolomon;
 	if (dtmf) delete dtmf;
 
@@ -638,25 +837,58 @@ void trx_start(void)
 	switch (progdefaults.btnAudioIOis) {
 #if USE_OSS
 	case SND_IDX_OSS:
-		scard = new SoundOSS(scDevice[0].c_str());
+		RXscard = new SoundOSS(scDevice[0].c_str());
+		TXscard = new SoundOSS(scDevice[0].c_str());
 		break;
 #endif
 #if USE_PORTAUDIO
 	case SND_IDX_PORT:
-		scard = new SoundPort(scDevice[0].c_str(), scDevice[1].c_str());
+		RXscard = new SoundPort(scDevice[0].c_str(), scDevice[1].c_str());
+		TXscard = new SoundPort(scDevice[0].c_str(), scDevice[1].c_str());
 		break;
 #endif
 #if USE_PULSEAUDIO
 	case SND_IDX_PULSE:
-		scard = new SoundPulse(scDevice[0].c_str());
+		try {
+			RXscard = new SoundPulse(scDevice[0].c_str());
+			if (!RXscard) break;
+
+			TXscard = new SoundPulse(scDevice[0].c_str());
+			if (!TXscard) break;
+
+// needed to open playback device in PaVolumeControl
+			TXscard->Open(O_WRONLY, current_TXsamplerate = 8000);
+			double buffer[1024];
+			for (int i = 0; i < 1024; buffer[i++] = 0);
+			TXscard->Write_stereo(buffer, buffer, 1024);
+
+			if (progdefaults.is_full_duplex)
+				TXsc_is_open = true;
+			else {
+				TXscard->Close();
+				TXsc_is_open = false;
+			}
+		} catch (const SndException& e) {
+			LOG_ERROR("%s", e.what());
+			if (RXscard) delete RXscard;
+			if (TXscard) delete TXscard;
+			RXscard = 0;
+			TXscard = 0;
+			reset_loop_msg = "Pulse Audio error:\n";
+			reset_loop_msg.append(e.what());
+			reset_loop_msg.append("\n\nIs the server running?");
+			REQ(show_reset_loop_alert);
+		}
 		break;
 #endif
 	case SND_IDX_NULL:
-		scard = new SoundNull;
+		RXscard = new SoundNull;
+		TXscard = new SoundNull;
 		break;
 	default:
 		abort();
 	}
+	current_RXsamplerate = current_TXsamplerate = 0;
 
 	ReedSolomon = new cRsId;
 	dtmf = new cDTMF;
@@ -724,9 +956,9 @@ void trx_close()
 	delete trx_sem;
 #endif
 
-	if (scard) {
-		delete scard;
-		scard = 0;
+	if (RXscard) {
+		delete RXscard;
+		RXscard = 0;
 	}
 	LOG_INFO("%s", "trx thread closed");
 }
